from typing import Iterable
from typing import IO

from . import json_iter
from . import jsonl_iter
from . import utils


def load(
    fp: IO,
    *,
    filetype=None
) -> Iterable:
    _iter = json_iter
    if filetype == 'json':
        pass
    elif filetype == 'jsonl':
        _iter = jsonl_iter
    else:
        if utils.is_jsonl(fp.name):
            _iter = jsonl_iter

    yield from _iter.load(fp)


def load_from_file(
        filename: str,
        *,
        filetype=None
        ):
    _iter = json_iter
    if filetype == 'json':
        pass
    elif filetype == 'jsonl':
        _iter = jsonl_iter
    else:
        if utils.is_jsonl(filename):
            _iter = jsonl_iter

    yield from _iter.load_from_file(filename)


def dump(
        in_iter_,
        fp: IO,
        *,
        filetype=None
        ):
    _iter = json_iter
    if filetype == 'json':
        pass
    elif filetype == 'jsonl':
        _iter = jsonl_iter
    else:
        if utils.is_jsonl(fp.name):
            _iter = jsonl_iter

    _iter.dump(in_iter_, fp)


def dump_to_file(
        in_iter_,
        filename: str,
        *,
        filetype=None
        ):
    _iter = json_iter
    if filetype == 'json':
        pass
    elif filetype == 'jsonl':
        _iter = jsonl_iter
    else:
        if utils.is_jsonl(filename):
            _iter = jsonl_iter

    _iter.dump_to_file(in_iter_, filename)


if __name__ == '__main__':
    files = [
        'tests/data/dict.json',
        'tests/data/array.json',
        'tests/data/array.jsonl',
    ]

    for f in files:
        print('reading {f} ...'.format(f=f))
        for i, o in enumerate(load_from_file(f)):
            print('{i}: {o}'.format(i=i, o=o))



import io
import itertools

import pytest

from sb_json_tools import jt_iter


@pytest.mark.parametrize("data,facit,filetype", [
    (1, "1", "json"),
    (1, "1\n", "jsonl"),
    ([1, 2], "[\n1,\n2\n]", "json"),
    ([1, 2], "1\n2\n", "jsonl"),
])
def test_dump_int_stringio(data, facit, filetype):
    out = io.StringIO()
    jt_iter.dump(data, out, filetype=filetype)
    assert out.getvalue() == facit

    out.seek(0)  # read it from the beginning
    out_iter = jt_iter.load(out, filetype=filetype)
    if isinstance(data, list):
        for i, f in itertools.zip_longest(out_iter, data):
            print("i = {i}".format(i=i))
            assert i is not None
            assert f is not None
            assert i == f
    else:
        for i in jt_iter.load(out, filetype=filetype):
            print("i = {i}".format(i=i))
            assert i == data



__version__ = '0.2.8'
import sys

import click

from sb_json_tools import jsonlib
from sb_json_tools import jt_val
from sb_json_tools import jt_iter


@click.group()
def cli():
    pass


@cli.command()
@click.option("--schema", "-s", help="Schema to use for validating.")
@click.argument("infile", type=click.File('r'))
@click.argument("outfile", type=click.File('w'))
def validate(infile, outfile, schema):
    """Validates a json-file with a schema (json-schema.org)."""
    # click.echo("Validating {0} with the schema in {1}.".format(infile, schema))
    schema_json = jsonlib.load_from_file(schema)
    data = jt_iter.load(infile)
    correct, errors = jt_val.validate(schema_json, data)
    jt_iter.dump(correct, outfile)
    if errors:
        # (out_root, out_ext) = os.path.splitext(outfile)
        # errors_filename = os.path.join(out_root, ".errors", out_ext)
        jt_iter.dump(errors, sys.stderr)
        # click.echo("ERRORs found!!!")
        # click.echo("Errors are written to {0}".format(errors_filename))
        click.Context.exit(len(errors))


@cli.command()
@click.argument('src', type=click.File('r'))
@click.argument('dst', type=click.File('w'))
def convert(src, dst):
    jt_iter.dump(
        jt_iter.load(src),
        dst
    )


if __name__ == '__main__':
    cli()



try:
    from ujson import dump, dumps, load, loads  # noqa: F401
except ImportError:
    from json import dump, dumps, load, loads  # noqa: F401


def load_from_file(filename: str):
    with open(filename, 'r') as fp:
        return load(fp)


def dump_to_file(obj, filename: str):
    with open(filename, 'w') as fp:
        return dump(obj, fp)



from sb_json_tools import jt_val as jv


schema = {
    "type": "object",
    "properties": {
        "int": {
            "type": "integer"
        },
        "num": {
            "type": "number"
        }
    },
    "required": ["int", "num"]
}


valid_data = [
    {"int": 1, "num": 3.4},
    {"int": 2.0, "num": 3.4},
    {"int": 2, "num": 4}
]


invalid_data = [
    {"int": 2.5, "num": 5.6}
]


def test_valid_data():
    for ok, error in jv.streaming_validate(schema, valid_data):
        assert ok is not None
        assert error is None


def test_invalid_data():
    for ok, error in jv.streaming_validate(schema, invalid_data):
        assert ok is None
        assert error is not None



from typing import Dict
from typing import IO
from typing import Iterable
from typing import Union

from sb_json_tools import jsonlib


def dump(data: Union[Dict, Iterable], fp: IO):
    if isinstance(data, dict):
        fp.write(jsonlib.dumps(data))
        fp.write('\n')
        return

    try:
        for obj in data:
            fp.write(jsonlib.dumps(obj))
            fp.write('\n')
    except TypeError:
        fp.write(jsonlib.dumps(data))
        fp.write('\n')


def load(fp: IO) -> Iterable:
    for line in fp:
        yield jsonlib.loads(line)


def load_from_file(filename: str):
    with open(filename, 'r') as fp:
        yield from load(fp)


def dump_to_file(obj, filename):
    with open(filename, 'w') as fp:
        dump(obj, fp)


if __name__ == '__main__':
    import sys
    data = [{'a': 1}, {'a': 2}]
    dump(data, sys.stdout)



from sb_json_tools import exceptions


class JsonValidatorException(exceptions.JsonToolsException):
    """
    Base exception for json_tools.val
    """

    def __init__(self, message):
        """
        A short description.

        A bit longer description.

            :param message: description

        Returns:
            type: description

        Raises:
            Exception: description

        """

        super().__init__(message)


class SchemaDefinitionException(JsonValidatorException):
    def __init__(self, message):
        super().__init__(message)


class ValidationException(JsonValidatorException):
    def __init__(self, message):
        super().__init__(message)



from json_tools.iter import json_iter


def main():
    data_source = json_iter.load_from_file('benches/data/skbl.json')
    def doc_update(doc):
        doc['_source']['lexiconName'] = 'skbl2'
        doc['_source']['lexiconOrder'] = 48
        return doc
    update_data = (doc_update(doc) for doc in data_source)
    json_iter.dump_to_file(update_data, 'benches/data/skbl2.json')


if __name__ == '__main__':
    main()



from sb_json_tools.jt_iter.utils import is_jsonl


def test_jsonl():
    filename = "test.jsonl"
    assert is_jsonl(filename)


def test_json():
    filename = "test.json"
    assert not is_jsonl(filename)



import io

from sb_json_tools.jt_iter import jsonl_iter as jiter


data = [
    {'a': 1},
    {'a': 2},
]

facit = '{"a": 1}\n{"a": 2}\n'


def gen_data():
    for i in data:
        yield i


def test_dump_dict_stringio():
    out = io.StringIO()
    jiter.dump(data[0], out)
    assert out.getvalue() == '{"a": 1}\n'

    out.seek(0)
    for i in jiter.load(out):
        print("i = {i}".format(i=i))
        assert i == data[0]


def test_dump_array_stringio():
    out = io.StringIO()
    jiter.dump(data, out)
    assert facit == out.getvalue()


def test_dump_gen_stringio():
    out = io.StringIO()
    jiter.dump(gen_data(), out)
    assert facit == out.getvalue()



from typing import Dict
from typing import IO
from typing import Iterable
from typing import Union

from sb_json_tools import jsonlib


def dump(data: Union[Dict, Iterable], fp: IO):
    """ Dump array to a file object.

    Parameters
    ----------
    fp :
        File object to write to. Must be writable.
    data :
        Iterable object to write.
    """
    if isinstance(data, dict):
        fp.write(jsonlib.dumps(data))
        return

    try:
        it = iter(data)
    except TypeError:
        fp.write(jsonlib.dumps(data))
        return

    fp.write('[\n')
    try:
        obj = next(it)
        fp.write(jsonlib.dumps(obj))
    except StopIteration:
        pass
    else:
        for v in it:
            fp.write(',\n')
            fp.write(jsonlib.dumps(v))
    fp.write('\n]')


def load(fp: IO) -> Iterable:
    # Determine type of data
    c = fp.read(1)
    if c != '[':
        fp.seek(0)
        yield jsonlib.load(fp)
    else:
        balance = 0
        start_idx = None
        chunk_size = 0
        while True:
            curr_idx = fp.tell()
            c = fp.read(1)
            # print(f'c = {c}')
            if not c:
                # print('End of file')

                break
            # chunk_size += 1
            if c == '{':
                balance += 1
                if balance == 1:
                    chunk_size = 1
                    start_idx = curr_idx
            elif c == '}':
                balance -= 1

                if balance == 0:
                    fp.seek(start_idx)
                    chunk = fp.read(chunk_size)
                    # print(f'read chunk "{chunk}"')
                    yield jsonlib.loads(chunk)
                    chunk_size = 0

            chunk_size += 1

        # print(f"balance = {balance}")
        # print(f"start_idx = {start_idx}")
        # print(f"chunk_size = {chunk_size}")
        if not start_idx:
            fp.seek(0)
            json_data = jsonlib.load(fp)
            if isinstance(json_data, list):
                for json_obj in json_data:
                    yield json_obj
            else:
                yield json_data


def load_eager(fp: IO):
    data = jsonlib.load(fp)
    if isinstance(data, list):
        for obj in data:
            yield obj
    else:
        return data


def load_from_file(filename: str):
    with open(filename, 'r') as fp:
        yield from load(fp)


def dump_to_file(filename, gen):
    with open(filename, 'w') as fp:
        return dump(gen, fp)


def main():
    files = [
        'tests/data/dict.json',
        'tests/data/array.json',
    ]

    for f in files:
        print('Reading {f} ...'.format(f=f))
        for obj in load_from_file(f):
            print('obj = {obj}'.format(obj=obj))
        print('')


if __name__ == '__main__':
    import sys
    data = [{'a': 1}, {'a': 2}]
    dump(data, sys.stdout)
    print('')
    main()
